import { useState, useEffect } from 'react'
import * as Popover from '@radix-ui/react-popover'
import { MagnifyingGlassIcon, XMarkIcon } from '@heroicons/react/24/outline'
import { isEqual } from 'lodash-es'

export default function TagSearchInput({ value, onChange, allTags = {}, placeholder = "输入标签名称搜索" }) {
  // value 现在是逗号分隔的标签 ID 列表
  const [searchQuery, setSearchQuery] = useState('')
  const [selectedTags, setSelectedTags] = useState([])
  const [isOpen, setIsOpen] = useState(false)
  const [availableTags, setAvailableTags] = useState({})
  const [filteredTags, setFilteredTags] = useState({})
  const [internalValue, setInternalValue] = useState(value || [])
  
  // 只在组件初始化或外部value变化时更新selectedTags
  useEffect(() => {
    if (isEqual(value, internalValue)) {
      setInternalValue(value || [])
      if (value) {
        const tagIds = value
        
        // 从所有可用标签中找出已选标签
        const selected = []
        Object.values(allTags).forEach(tagGroup => {
          tagGroup.forEach(tag => {
            if (tagIds.includes(tag.id)) {
              selected.push(tag)
            }
          })
        })
        
        setSelectedTags(selected)
      } else {
        setSelectedTags([])
      }
    }
  }, [value, allTags])

  // 使用传入的标签数据
  useEffect(() => {
    setAvailableTags(allTags || {})
  }, [allTags])

  // 根据搜索词过滤标签
  useEffect(() => {
    if (!searchQuery) {
      setFilteredTags(availableTags)
      return
    }
    
    const query = searchQuery.toLowerCase()
    const filtered = {}
    
    Object.entries(availableTags).forEach(([catalog, tags]) => {
      const matchingTags = tags.filter(tag => 
        tag.name.toLowerCase().includes(query)
      )
      
      if (matchingTags.length > 0) {
        filtered[catalog] = matchingTags
      }
    })
    
    setFilteredTags(filtered)
  }, [searchQuery, availableTags])

  const handleSelectTag = (tag) => {
    // 检查标签是否已经选中
    const isSelected = selectedTags.some(t => t.id === tag.id)
    
    let newSelectedTags
    if (isSelected) {
      // 如果已选中，则移除该标签（反选）
      newSelectedTags = selectedTags.filter(t => t.id !== tag.id)
    } else {
      // 如果未选中，则添加该标签
      newSelectedTags = [...selectedTags, tag]
    }
    
    setSelectedTags(newSelectedTags)
    
    // 直接更新内部值并触发onChange，使用tag_ids
    const newValue = newSelectedTags.map(t => t.id)
    setInternalValue(newValue)
    onChange(newValue)
    
    // 不清空搜索框，保留搜索内容
  }

  const handleRemoveTag = (tagId) => {
    const newSelectedTags = selectedTags.filter(tag => tag.id !== tagId)
    setSelectedTags(newSelectedTags)
    // 直接更新内部值并触发onChange，使用tag_ids
    const newValue = newSelectedTags.map(t => t.id)
    setInternalValue(newValue)
    onChange(newValue)
  }

  return (
    <div className="relative">
      <Popover.Root open={isOpen} onOpenChange={setIsOpen} modal={false}>
        <Popover.Trigger asChild>
          <div className="flex flex-wrap gap-2 min-h-[2.5rem] p-2 border border-gray-300 rounded-md bg-white cursor-pointer">
            {selectedTags.map(tag => (
              <div 
                key={tag.id} 
                className="flex items-center bg-indigo-100 text-indigo-800 text-sm rounded-full px-3 py-1 cursor-pointer"
                onClick={() => handleRemoveTag(tag.id)}
              >
                <span>{tag.name}</span>
                <button
                  type="button"
                  onClick={(e) => {
                    e.stopPropagation() // 阻止事件冒泡到父元素
                    handleRemoveTag(tag.id)
                  }}
                  className="ml-1.5 text-indigo-600 hover:text-indigo-800 focus:outline-none"
                >
                  <span className="sr-only">移除标签 {tag.name}</span>
                  <XMarkIcon className="h-3.5 w-3.5" />
                </button>
              </div>
            ))}
            
            {selectedTags.length === 0 && (
              <div className="text-gray-400 text-sm">{placeholder}</div>
            )}
          </div>
        </Popover.Trigger>
        
        {/* 保留搜索图标，但不作为触发器 */}
        <div className="absolute right-2 top-1/2 -translate-y-1/2 text-gray-400 pointer-events-none">
          <MagnifyingGlassIcon className="h-5 w-5" />
        </div>
        <Popover.Portal>
          <Popover.Content 
            className="bg-white rounded-md shadow-lg p-4 w-72 max-w-[calc(100vw-2rem)] z-50"
            sideOffset={5}
          >
            <div className="space-y-4">
              <div className="relative">
                <label htmlFor="tag-search" className="sr-only">
                  搜索标签
                </label>
                <input
                  type="text"
                  id="tag-search"
                  className="block w-full border-gray-300 rounded-md shadow-sm focus:ring-indigo-500 focus:border-indigo-500 sm:text-sm pr-8"
                  placeholder="搜索标签..."
                  value={searchQuery}
                  onChange={(e) => setSearchQuery(e.target.value)}
                  autoFocus
                />
                {searchQuery && (
                  <button
                    type="button"
                    onClick={() => setSearchQuery('')}
                    className="absolute right-2 top-1/2 -translate-y-1/2 text-gray-400 hover:text-gray-600"
                    aria-label="清空搜索"
                  >
                    <XMarkIcon className="h-4 w-4" />
                  </button>
                )}
              </div>
              
              <div className="max-h-60 overflow-y-auto">
                {Object.keys(filteredTags).length > 0 ? (
                  <div className="space-y-4">
                    {Object.entries(filteredTags).map(([catalog, tags]) => (
                      <div key={catalog || 'uncategorized'} className="space-y-2">
                        <h3 className="text-sm font-bold text-gray-700">
                          {catalog || '未分类'}
                        </h3>
                        <div className="space-y-1">
                          {tags.map(tag => {
                            const isSelected = selectedTags.some(t => t.id === tag.id)
                            return (
                              <button
                                key={tag.id}
                                type="button"
                                onClick={() => handleSelectTag(tag)}
                                // 已选中的标签也可以点击，用于反选
                                className={`w-full text-left px-3 py-2 text-sm rounded-md ${
                                  isSelected
                                    ? 'bg-indigo-100 text-indigo-800'
                                    : 'text-gray-700 hover:bg-gray-100'
                                }`}
                              >
                                {tag.name}
                                {tag.images_count > 0 && (
                                  <span className="ml-2 text-xs text-gray-500">
                                    ({tag.images_count})
                                  </span>
                                )}
                              </button>
                            )
                          })}
                        </div>
                      </div>
                    ))}
                  </div>
                ) : (
                  <div className="text-center py-4 text-sm text-gray-500">
                    {searchQuery ? '没有找到匹配的标签' : '没有可用的标签'}
                  </div>
                )}
              </div>
            </div>
            <Popover.Arrow className="fill-white" />
          </Popover.Content>
        </Popover.Portal>
      </Popover.Root>
    </div>
  )
}
