import { useState, useMemo } from 'react'
import { Link, useForm } from '@inertiajs/react'
import * as Dialog from '@radix-ui/react-dialog'
import * as Form from '@radix-ui/react-form'
import { TagIcon, XMarkIcon, MagnifyingGlassIcon } from '@heroicons/react/24/outline'

export default function ComTagsIndex({ 
  title, 
  description, 
  tags, 
  auth, 
  isAdmin = false, 
  path, 
  errors = {} 
}) {
  const [isCreateModalOpen, setIsCreateModalOpen] = useState(false)
  const [isEditModalOpen, setIsEditModalOpen] = useState(false)
  const [editingTag, setEditingTag] = useState(null)
  const [searchQuery, setSearchQuery] = useState('')
  
  // Get all catalog names
  const catalogNames = Object.keys(tags);
  
  // Filter tags based on search query
  const filteredTags = useMemo(() => {
    if (!searchQuery.trim()) return tags;
    
    const query = searchQuery.toLowerCase().trim();
    const result = {};
    
    Object.keys(tags).forEach(catalog => {
      const filteredCatalogTags = tags[catalog].filter(tag => 
        tag.name.toLowerCase().includes(query)
      );
      
      if (filteredCatalogTags.length > 0) {
        result[catalog] = filteredCatalogTags;
      }
    });
    
    return result;
  }, [tags, searchQuery]);
  
  const { data: createData, setData: setCreateData, post: createTag, processing: createProcessing, reset: resetCreate } = useForm({
    name: '',
    catalog: ''
  })

  const { data: editData, setData: setEditData, patch: updateTag, processing: editProcessing, reset: resetEdit } = useForm({
    name: '',
    catalog: ''
  })

  const handleCreateSubmit = (e) => {
    e.preventDefault()
    createTag(`${path}`, {
      onSuccess: () => {
        setIsCreateModalOpen(false)
        resetCreate()
      },
    })
  }

  const handleEditSubmit = (e) => {
    e.preventDefault()
    updateTag(`${path}/${editingTag.id}`, {
      onSuccess: () => {
        setIsEditModalOpen(false)
        setEditingTag(null)
        resetEdit()
      },
    })
  }

  const openEditModal = (tag) => {
    setEditingTag(tag)
    setEditData({ 
      name: tag.name,
      catalog: tag.catalog || ''
    })
    setIsEditModalOpen(true)
  }

  return (
    <div className="bg-white shadow overflow-hidden sm:rounded-lg">
      <div className="px-4 py-5 sm:px-6 flex flex-col sm:flex-row justify-between items-start sm:items-center gap-4">
        <div>
          <h1 className="text-2xl font-bold text-gray-900">{title}</h1>
          <p className="mt-1 max-w-2xl text-sm text-gray-500">
            {description}
          </p>
        </div>
        
        <div className="flex flex-col sm:flex-row gap-3 w-full sm:w-auto">
          {/* Search input */}
          <div className="relative flex-grow sm:max-w-xs">
            <div className="absolute inset-y-0 left-0 pl-3 flex items-center pointer-events-none">
              <MagnifyingGlassIcon className="h-5 w-5 text-gray-400" aria-hidden="true" />
            </div>
            <input
              type="text"
              value={searchQuery}
              onChange={(e) => setSearchQuery(e.target.value)}
              placeholder="搜索标签..."
              className="block w-full pl-10 pr-3 py-2 border border-gray-300 rounded-md leading-5 bg-white placeholder-gray-500 focus:outline-none focus:placeholder-gray-400 focus:ring-1 focus:ring-indigo-500 focus:border-indigo-500 sm:text-sm"
            />
          </div>
          
          {auth && isAdmin && (
            <button
              onClick={() => setIsCreateModalOpen(true)}
              className="inline-flex items-center px-4 py-2 border border-transparent text-sm font-medium rounded-md shadow-sm text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500"
            >
              创建新标签
            </button>
          )}
        </div>
      </div>

      <div className="border-t border-gray-200">
        {Object.keys(filteredTags).length > 0 ? (
          <div className="px-4 py-5 space-y-8">
            {Object.keys(filteredTags).map((catalog) => (
              <div key={catalog} className="pb-4">
                <h3 className="text-lg font-medium text-gray-900 mb-4">
                  {catalog || '未分类'} <span className="text-sm text-gray-500">({filteredTags[catalog].length})</span>
                </h3>
                <div className="grid grid-cols-1 sm:grid-cols-2 md:grid-cols-3 lg:grid-cols-4 gap-4">
                  {filteredTags[catalog].map((tag) => (
                    <div
                      key={tag.id}
                      className="bg-gray-50 rounded-lg p-4 py-2 hover:bg-gray-100 transition-colors flex flex-col"
                      >
                        <div className="flex justify-between items-start">
                          <div className="flex items-center">
                            <div className="flex-shrink-0 mr-2">
                              <TagIcon className="h-5 w-5 text-indigo-500" />
                            </div>
                            <div>
                              <span className="text-sm font-medium text-gray-900">{tag.name}</span>
                            </div>
                          </div>
                          <Link
                            href={`${path}/${tag.id}/images`}
                            className="text-xs font-medium text-gray-700 bg-gray-200 hover:bg-gray-300 px-2 py-1 rounded-full transition-colors"
                          >
                            {tag.images_count}
                          </Link>
                        </div>
                        
                        {isAdmin && <div className="!mt-2 flex space-x-2 justify-end mt-auto pt-2 border-t border-gray-200">
                          <button
                            onClick={() => openEditModal(tag)}
                            className="text-xs text-indigo-600 hover:text-indigo-900"
                          >
                            编辑
                          </button>
                          <Link
                            href={`${path}/${tag.id}`}
                            method="delete"
                            as="button"
                            data={{ confirm: "确定要删除这个标签吗？这将从所有关联的图片中移除该标签。" }}
                            className="text-xs text-red-600 hover:text-red-900"
                          >
                            删除
                          </Link>
                        </div>}
                      </div>
                    ))}
                  </div>
              </div>
            ))}
          </div>
        ) : (
          <div className="text-center py-12">
            <TagIcon className="mx-auto h-12 w-12 text-gray-400" />
            <h3 className="mt-2 text-sm font-medium text-gray-900">
              {searchQuery ? '没有匹配的标签' : '没有标签'}
            </h3>
            <p className="mt-1 text-sm text-gray-500">
              {searchQuery 
                ? '尝试使用不同的搜索词。' 
                : isAdmin ? '创建一个新标签开始使用。' : '目前没有可用的标签。'}
            </p>
            {auth && isAdmin && !searchQuery && (
              <div className="mt-6">
                <button
                  onClick={() => setIsCreateModalOpen(true)}
                  className="inline-flex items-center px-4 py-2 border border-transparent shadow-sm text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500"
                >
                  创建新标签
                </button>
              </div>
            )}
          </div>
        )}
      </div>

      {/* Create Tag Modal */}
      {isAdmin && (
        <Dialog.Root open={isCreateModalOpen} onOpenChange={setIsCreateModalOpen}>
          <Dialog.Portal>
            <Dialog.Overlay className="fixed inset-0 bg-black/30" />
            <Dialog.Content className="fixed top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 bg-white rounded-lg shadow-lg p-6 w-full max-w-md">
              <Dialog.Title className="text-lg font-medium text-gray-900">
                创建新标签
              </Dialog.Title>
              <Dialog.Description className="mt-2 text-sm text-gray-500">
                添加一个新标签用于图片分类。
              </Dialog.Description>

              <Form.Root className="mt-4" onSubmit={handleCreateSubmit}>
                <Form.Field name="name" className="mb-4">
                  <div className="flex items-baseline justify-between">
                    <Form.Label className="text-sm font-medium text-gray-700">
                      标签名称
                    </Form.Label>
                    <Form.Message
                      className="text-xs text-red-500"
                      match="valueMissing"
                    >
                      请输入标签名称
                    </Form.Message>
                    {errors.name && (
                      <p className="text-xs text-red-500">{errors.name}</p>
                    )}
                  </div>
                  <Form.Control asChild>
                    <input
                      type="text"
                      value={createData.name}
                      onChange={(e) => setCreateData({ ...createData, name: e.target.value })}
                      className="mt-1 block w-full rounded-md border-gray-300 shadow-sm focus:border-indigo-500 focus:ring-indigo-500 sm:text-sm"
                      required
                    />
                  </Form.Control>
                </Form.Field>

                <Form.Field name="catalog" className="mb-4">
                  <div className="flex items-baseline justify-between">
                    <Form.Label className="text-sm font-medium text-gray-700">
                      分类（可选）
                    </Form.Label>
                    {errors.catalog && (
                      <p className="text-xs text-red-500">{errors.catalog}</p>
                    )}
                  </div>
                  <Form.Control asChild>
                    <input
                      type="text"
                      value={createData.catalog}
                      onChange={(e) => setCreateData({ ...createData, catalog: e.target.value })}
                      className="mt-1 block w-full rounded-md border-gray-300 shadow-sm focus:border-indigo-500 focus:ring-indigo-500 sm:text-sm"
                    />
                  </Form.Control>
                </Form.Field>

                <div className="mt-5 sm:mt-6 sm:grid sm:grid-cols-2 sm:gap-3 space-y-2">
                  <button
                    type="button"
                    onClick={() => setIsCreateModalOpen(false)}
                    className="inline-flex justify-center w-full rounded-md border border-gray-300 shadow-sm px-4 py-2 bg-white text-base font-medium text-gray-700 hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500 sm:text-sm"
                  >
                    取消
                  </button>
                  <button
                    type="submit"
                    disabled={createProcessing}
                    className="inline-flex justify-center w-full rounded-md border border-transparent shadow-sm px-4 py-2 bg-indigo-600 text-base font-medium text-white hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500 sm:text-sm disabled:opacity-50"
                  >
                    创建
                  </button>
                </div>
              </Form.Root>

              <Dialog.Close asChild>
                <button
                  className="absolute top-4 right-4 text-gray-400 hover:text-gray-500"
                  aria-label="Close"
                >
                  <XMarkIcon className="h-5 w-5" />
                </button>
              </Dialog.Close>
            </Dialog.Content>
          </Dialog.Portal>
        </Dialog.Root>
      )}

      {/* Edit Tag Modal */}
      {isAdmin && (
        <Dialog.Root open={isEditModalOpen} onOpenChange={setIsEditModalOpen}>
          <Dialog.Portal>
            <Dialog.Overlay className="fixed inset-0 bg-black/30" />
            <Dialog.Content className="fixed top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 bg-white rounded-lg shadow-lg p-6 w-full max-w-md">
              <Dialog.Title className="text-lg font-medium text-gray-900">
                编辑标签
              </Dialog.Title>
              <Dialog.Description className="mt-2 text-sm text-gray-500">
                更新标签信息。
              </Dialog.Description>

              <Form.Root className="mt-4" onSubmit={handleEditSubmit}>
                <Form.Field name="name" className="mb-4">
                  <div className="flex items-baseline justify-between">
                    <Form.Label className="text-sm font-medium text-gray-700">
                      标签名称
                    </Form.Label>
                    <Form.Message
                      className="text-xs text-red-500"
                      match="valueMissing"
                    >
                      请输入标签名称
                    </Form.Message>
                    {errors.name && (
                      <p className="text-xs text-red-500">{errors.name}</p>
                    )}
                  </div>
                  <Form.Control asChild>
                    <input
                      type="text"
                      value={editData.name}
                      onChange={(e) => setEditData({ ...editData, name: e.target.value })}
                      className="mt-1 block w-full rounded-md border-gray-300 shadow-sm focus:border-indigo-500 focus:ring-indigo-500 sm:text-sm"
                      required
                    />
                  </Form.Control>
                </Form.Field>

                <Form.Field name="catalog" className="mb-4">
                  <div className="flex items-baseline justify-between">
                    <Form.Label className="text-sm font-medium text-gray-700">
                      分类（可选）
                    </Form.Label>
                    {errors.catalog && (
                      <p className="text-xs text-red-500">{errors.catalog}</p>
                    )}
                  </div>
                  <Form.Control asChild>
                    <input
                      type="text"
                      value={editData.catalog}
                      onChange={(e) => setEditData({ ...editData, catalog: e.target.value })}
                      className="mt-1 block w-full rounded-md border-gray-300 shadow-sm focus:border-indigo-500 focus:ring-indigo-500 sm:text-sm"
                    />
                  </Form.Control>
                </Form.Field>

                <div className="mt-5 sm:mt-6 sm:grid sm:grid-cols-2 sm:gap-3 space-y-2">
                  <button
                    type="button"
                    onClick={() => {
                      setIsEditModalOpen(false)
                      setEditingTag(null)
                    }}
                    className="inline-flex justify-center w-full rounded-md border border-gray-300 shadow-sm px-4 py-2 bg-white text-base font-medium text-gray-700 hover:bg-gray-50 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500 sm:text-sm"
                  >
                    取消
                  </button>
                  <button
                    type="submit"
                    disabled={editProcessing}
                    className="inline-flex justify-center w-full rounded-md border border-transparent shadow-sm px-4 py-2 bg-indigo-600 text-base font-medium text-white hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500 sm:text-sm disabled:opacity-50"
                  >
                    更新
                  </button>
                </div>
              </Form.Root>

              <Dialog.Close asChild>
                <button
                  className="absolute top-4 right-4 text-gray-400 hover:text-gray-500"
                  aria-label="Close"
                >
                  <XMarkIcon className="h-5 w-5" />
                </button>
              </Dialog.Close>
            </Dialog.Content>
          </Dialog.Portal>
        </Dialog.Root>
      )}
    </div>
  )
}
